#include <iostream>
#include <stdio.h>
#include <string>
#include <algorithm>
#include <map>
#include <vector>
#include <iomanip>
#include <stack>


using namespace std;
#define SQR(X) ((X)*(X))


template<class Type>
struct VectorTemplateT
{
	Type x, y;

	inline VectorTemplateT(Type xx = 0, Type yy = 0): x(xx), y(yy) {  };

	template<class OldType>
	inline VectorTemplateT(const VectorTemplateT<OldType>& np)
	{
		x = (OldType)np.x;
		y = (OldType)np.y;
	}

	inline double Len() const
	{
		return sqrt((double)Len2());
	}

	inline Type Len2() const
	{
		return SQR(x) + SQR(y);
	}

	inline VectorTemplateT<Type> operator - (const VectorTemplateT<Type>& a) const
	{
		return VectorTemplateT<Type>(x - a.x, y - a.y);
	}

	inline VectorTemplateT<Type> operator + (const VectorTemplateT<Type>& a) const
	{
		return VectorTemplateT<Type>(x + a.x, y + a.y);
	}

	inline Type operator * (const VectorTemplateT<Type>& a) const
	{
		return x*a.x + y*a.y;
	}

	inline VectorTemplateT<double> operator * (double s) const
	{
		return VectorTemplateT<double>(x*s, y*s);
	}
};


template<class Type>
inline istream& operator >> (istream& in, VectorTemplateT<Type>& v)
{
	in >> v.x >> v.y;
	return in;
}


template<class Type>
inline ostream& operator << (ostream& out, VectorTemplateT<Type>& v)
{
	out << fixed << setprecision(2) << v.x << " " << v.y << endl;
	return out;
}


#define PointTemplateT VectorTemplateT
typedef VectorTemplateT<long long> VectorT;
typedef VectorT PointT;


inline long long norm2(const PointT& a, const PointT& b)
{
	return SQR(a.x - b.x) + SQR(a.y - b.y);
}


inline double norm(const PointT& a, const PointT& b)
{
	return sqrt((double)norm2(a, b));
}


struct EventT
{
	enum EventType
	{
		Tangent,
		Ending,
		Beginning,
	} type;
	double t;
	int n;

	//EventT() {}
	EventT(EventType et, double tt, int nn): type(et), t(tt), n(nn) {  }
};


inline void Assert(bool cond, const string& msg)
{
	if (!cond) {
		cout << msg << endl;
		cerr << msg << endl;
		exit(216);
	}
}


struct CircleT
{
	PointT p;
	long long r, g;
	int n;
	vector<EventT> e;
	
	CircleT(const PointT& pp = PointT(), long long rr = 0, long long gg = 0, int nn = 0): p(pp), r(rr), g(gg), n(nn) {  }

	void CalculateEvent(const PointT& s, const VectorT& v)
	{
		long long A = v.Len2();
		long long B = 2 * ((s - p) * v);
		long long C = (s - p).Len2() - SQR(r);
		long long D = SQR(B) - 4*A*C;
		if (D < 0) {
			/* nothing to do */
		}
		if (D == 0) {
			double t = -B / (double)(2*A);
			e.push_back(EventT(EventT::Tangent, t, n));
		}
		if (D > 0) {
			double sd = sqrt((double)D);
			double t1 = (-B - sd) / (double)(2*A);
			double t2 = (-B + sd) / (double)(2*A);
			Assert(t1 < t2, "t1 is not less than t2!");
			e.push_back(EventT(EventT::Beginning, t1, n));
			e.push_back(EventT(EventT::Ending, t2, n));
		}
	}
};


int intersection_point_quantity(const CircleT& c1, const CircleT& c2)
{
	long long d2 = norm2(c1.p, c2.p);
	long long rr_outer = SQR(c1.r + c2.r);
	long long rr_inner = SQR(c1.r - c2.r);
	return d2 > rr_outer || d2 < rr_inner && rr_inner != 0 ? 0 : d2 == rr_outer || d2 == rr_inner && rr_inner != 0 ? 1 : 2;
}


inline istream& operator >> (istream& in, CircleT& c)
{
	in >> c.p >> c.r >> c.g;
	return in;
}


inline void check_input_data(const vector<CircleT>& list)
{
	char msg[1000];
	for (int i = 0; i < list.size(); i++) {
		for (int j = i+1; j < list.size(); j++) {
			sprintf(msg, "Circles #%d and %d intersect", i, j);
			Assert(intersection_point_quantity(list[i], list[j]) < 2, msg);
		}
	}
}


const double eps = 1e-8;


struct EventComparatorT
{
	const vector<CircleT>& c;

	EventComparatorT(const vector<CircleT>& cc): c(cc) {  };

	bool operator () (const EventT& e1, const EventT& e2)
	{
		if (abs(e1.t - e2.t) > eps) return e1.t < e2.t;
		if (e1.type != e2.type) return e1.type == EventT::Ending;
		return e1.type == EventT::Ending ? c[e1.n].r < c[e2.n].r : c[e1.n].r > c[e2.n].r;
	}
};


vector<CircleT> circles;
int n;
double m;
PointT s;
VectorT v;
int r, g;
vector<EventT> events;
stack<EventT> event_order;

void test() {}

int main()
{
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
	
	cin >> s >> v >> m;
	cin >> r >> g;
	circles.push_back(CircleT(PointT(0, 0), r, g, 0));
	circles[0].CalculateEvent(s, v);
	Assert((circles[0].p - s).Len2() <= SQR(r), "S point is out of G0");

	cin >> n; n++;
	circles.resize(n);

	for (int i = 1; i < n; i++)
	{
		cin >> circles[i];
		circles[i].n = i;
		circles[i].CalculateEvent(s, v);
	}

	//check_input_data(circles);

	for (int i = 0; i < circles.size(); i++) {
		for (int j = 0; j < circles[i].e.size(); j++) {
			const EventT& e = circles[i].e[j];
			if(e.type == EventT::Beginning || e.type == EventT::Ending) {
				events.push_back(e);
			}
		}
	}

	EventComparatorT c(circles);
	sort(events.begin(), events.end(), c);
	event_order.push(events[0]);
	PointTemplateT<double> ss(s);
	PointTemplateT<double> ans(s);

	for (int i = 1; i < events.size(); i++) {
		const EventT& prev_event = event_order.top();
		const EventT& curr_event = events[i];
		if (curr_event.t > 0) {
			double curr_dist = (ans - (ss + v*curr_event.t)).Len();
			double req = curr_dist * circles[prev_event.n].g;
			if (req <= m) {
				m -= req;
				ans = ss + v*curr_event.t;
			} else {
				ans = ans + v * ((m / req) * (curr_dist / v.Len()));
				break;
			}
		}
		if (curr_event.type == EventT::Beginning) {
			event_order.push(curr_event);
		}
		if (curr_event.type == EventT::Ending) {
			event_order.pop();
		}
	}

	cout << ans;

	fclose(stdin);
	fclose(stdout);
	return 0;
}
